#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
from WikiExtractor import WikiExtractor
from urllib2 import unquote
from collections import namedtuple
from pprint import pprint

def clean(text, we = WikiExtractor()):
    doc = namedtuple('Doc', ['text'])
    doc.text = text
    doc.text = we._WikiExtractor__clean(doc).text
    text = we._WikiExtractor__compact(doc).text
    text = text.encode('utf-8')
    link_dictionary = doc.link_dictionary
    new = []
    for line in text:
        if line and line[0] not in ['=', ';', '*']:
            line = line.replace('()', '')
            new.append(line)
    return ''.join(new).decode('utf-8'), link_dictionary
   



# -*- coding: utf-8 -*-

stop_words = [
'ign',
'a',
'aby',
'ach',
'acz',
'aczkolwiek',
'aj',
'albo',
'ale',
'ależ',
'ani',
'aż',
'bardziej',
'bardzo',
'bez',
'bo',
'bowiem',
'by',
'byli',
'bynajmniej',
'być',
'był',
'była',
'było',
'były',
'będzie',
'będą',
'cali',
'cała',
'cały',
'ci',
'cię',
'ciebie',
'co',
'cokolwiek',
'coś',
'czasami',
'czasem',
'czemu',
'czy',
'czyli',
'daleko',
'dla',
'dlaczego',
'dlatego',
'do',
'dobrze',
'dokąd',
'dość',
'dużo',
'dwa',
'dwaj',
'dwie',
'dwoje',
'dziś',
'dzisiaj',
'gdy',
'gdyby',
'gdyż',
'gdzie',
'gdziekolwiek',
'gdzieś',
'go',
'i',
'ich',
'ile',
'im',
'inna',
'inne',
'inny',
'innych',
'iż',
'ja',
'ją',
'jak',
'jakaś',
'jakby',
'jaki',
'jakichś',
'jakie',
'jakiś',
'jakiż',
'jakkolwiek',
'jako',
'jakoś',
'je',
'jeden',
'jedna',
'jedno',
'jednak',
'jednakże',
'jego',
'jej',
'jemu',
'jest',
'jestem',
'jeszcze',
'jeśli',
'jeżeli',
'już',
'ją',
'każdy',
'kiedy',
'kilka',
'kimś',
'kto',
'ktokolwiek',
'ktoś',
'która',
'które',
'którego',
'której',
'który',
'których',
'którym',
'którzy',
'ku',
'lat',
'lecz',
'lub',
'ma',
'mają',
'mało',
'mam',
'mi',
'mimo',
'między',
'mną',
'mnie',
'mogą',
'moi',
'moim',
'moja',
'moje',
'może',
'możliwe',
'można',
'mój',
'mu',
'musi',
'my',
'na',
'nad',
'nam',
'nami',
'nas',
'nasi',
'nasz',
'nasza',
'nasze',
'naszego',
'naszych',
'natomiast',
'natychmiast',
'nawet',
'nią',
'nic',
'nich',
'nie',
'niech',
'niego',
'niej',
'niemu',
'nigdy',
'nim',
'nimi',
'niż',
'no',
'o',
'obok',
'od',
'około',
'on',
'ona',
'one',
'oni',
'ono',
'oraz',
'oto',
'owszem',
'pan',
'pana',
'pani',
'po',
'pod',
'podczas',
'pomimo',
'ponad',
'ponieważ',
'powinien',
'powinna',
'powinni',
'powinno',
'poza',
'prawie',
'przecież',
'przed',
'przede',
'przedtem',
'przez',
'przy',
'roku',
'również',
'sam',
'sama',
'są',
'się',
'skąd',
'sobie',
'sobą',
'sposób',
'swoje',
'ta',
'tak',
'taka',
'taki',
'takie',
'także',
'tam',
'te',
'tego',
'tej',
'ten',
'teraz',
'też',
'to',
'tobą',
'tobie',
'toteż',
'trzeba',
'tu',
'tutaj',
'twoi',
'twoim',
'twoja',
'twoje',
'twym',
'twój',
'ty',
'tych',
'tylko',
'tym',
'u',
'w',
'wam',
'wami',
'was',
'wasz',
'wasza',
'wasze',
'we',
'według',
'wiele',
'wielu',
'więc',
'więcej',
'wszyscy',
'wszystkich',
'wszystkie',
'wszystkim',
'wszystko',
'wtedy',
'wy',
'właśnie',
'z',
'za',
'zapewne',
'zawsze',
'ze',
'zł',
'znowu',
'znów',
'został',
'żaden',
'żadna',
'żadne',
'żadnych',
'że',
'żeby',
]



#!/usr/bin/env python
#encoding: utf-8
import sys
import os
import errno
from itertools import izip
from collections import defaultdict
from random import shuffle
from urllib import quote_plus

from config import tests_path, predicates, type_restrictions
from sparql_access import select_all, select_entities_of_type_in_relation
from sentence_classifier import SentenceClassifier
from article_access import get_article, prepare_articles
from language_tools import LanguageToolsFactory

if __name__ == '__main__':
    lt = LanguageToolsFactory.get_language_tools()
    predicate = ''
    test_data_limit = 100
    try:
        os.makedirs(tests_path + '%s' % predicate)
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise
    entities_f = open(tests_path + '%s/entities' % predicate, 'w')
    values_f = open(tests_path + '%s/values' % predicate, 'w')
    articles_f = open(tests_path + '%s/articles' % predicate, 'w')
    if predicate in type_restrictions:
        names = select_entities_of_type_in_relation(
            type_restrictions[predicate], predicate
        )
    else:
        names = select_all({'p': predicate})
    shuffle(names)
    names = names[: test_data_limit]
    subjects, objects = zip(*list(names))
    values = defaultdict(list)
    for subject, value in names:
        values[subject].append(value)
    prepare_articles(subjects)
    for subject, value in values.iteritems():
        try:
            article = get_article(subject)
        except:
            continue
        print >>articles_f, subject, lt.prepare_value(value[0], predicate)
        for sentence in article:
            sentence = [word.segment for word in sentence]
            print >>articles_f, ' '.join(sentence)
        print >>articles_f
        print >>entities_f, subject
        print >>values_f, subject, value[0].replace(' ', '_').encode('utf-8')




#encoding: utf-8
import os
import sys
from codecs import open as copen
from subprocess import Popen, PIPE
from itertools import izip
from locale import atof, setlocale, LC_NUMERIC
from urllib import quote_plus

from config import models_cache_path, verbose, numeric_predicates, use_parser, use_wordnet, evaluation_mode, save_to_cache
from language_tools import LanguageToolsFactory, is_numeric
from collect_entities import entities_types
from candidates_selector import CandidatesSelector

lt = LanguageToolsFactory.get_language_tools()

class ValueExtractor:
    def __init__(self, predicate, training_data):
        self.predicate = predicate
        self.predominant_types = map(
            lambda t: t.split('/')[-1], 
            CandidatesSelector.get_predominant_types(predicate, False)
        )
        self.model_filename = 'crfmodel-%s' % predicate
        self.features_train_filename = 'features_train'
        self.features_tag_filename = 'features_tag'
        try:
            open(models_cache_path % self.model_filename)
        except IOError:
            self.train(training_data)
        
    def extract_features(self, sentence, i, window_size=3):
        def recent_year(word):
            try:
                return 1990 <= int(word) <= 2012
            except ValueError:
                return False
        
        features = {}
        for j in xrange(-window_size, window_size + 1):
            if 0 <= i + j < len(sentence):
                word = sentence[i + j]
                lemma = word.lemma
                if is_numeric(lemma):
                    lemma = str(int(atof(lemma)))
                segment = word.segment
                tag = word.tag
                word_features = {
                    'segment': segment,
                    'tag': lt.get_tag(tag),
                    'case': lt.get_case(tag),
                    'number': lt.get_number(tag),
                    'gender': lt.get_gender(tag),
                    'person': lt.get_person(tag),
                    'aspect': lt.get_aspect(tag),
                    'lemma': lemma,
                    'recent_year': str(int(recent_year(lemma))),
                    'alldigits': str(int(lemma.isdigit())),
                    'allalpha': str(int(lemma.decode('utf-8').isalpha())),
                    'starts_with_capital': str(int(lemma.decode('utf-8')[0].isupper())),
                    'segm_starts_with_capital': str(int(segment.decode('utf-8')[0].isupper())),
                    'numeric': str(int(is_numeric(lemma))),
                    'len': str(len(lemma)),
                }
                if use_parser:
                    word_features['parse'] = word.parse
                if use_wordnet:
                    word_features['synset'] = get_hypernym(word)
                for name, feature in word_features.iteritems():
                    features['%d%s' % (j, name)] = feature
        return features
        
    def save_features_to_file(self, filename, sentences, selected_values=None):
        f = copen(models_cache_path % filename, 'w', 'utf-8')
        for i, sentence in enumerate(sentences):
            for j, word in enumerate(sentence):
                if selected_values:
                    values = selected_values[i]
                    cls = int(any(value.lower() in [word.lemma.lower(), word.segment.lower()] for value in values))
                    print >>f, '%d\t' % cls,
                for name, value in self.extract_features(sentence, j).iteritems():
                    print >>f, ('%s=%s\t' % (name, value)).decode('utf-8'),
                print >>f
            print >>f
        
    def train(self, training_data):
        sentences, selected_values = zip(*training_data)
        self.save_features_to_file(self.features_train_filename, sentences, selected_values)
        command = 'crfsuite learn -m %s %s' % (models_cache_path % self.model_filename, models_cache_path % self.features_train_filename)
        p = Popen(command, stdout=PIPE, stdin=PIPE, stderr=PIPE, shell=True)
        p.communicate()
        if verbose:
            self.print_most_informative_features()
        
    def print_most_informative_features(self, n=100):
        command = 'crfsuite dump %s' % (models_cache_path % self.model_filename)
        p = Popen(command, stdout=PIPE, stdin=PIPE, stderr=PIPE, shell=True)
        out, _ = p.communicate()
        out = out.split('\n')[:-3]
        start = out.index('STATE_FEATURES = {')
        feature_weights = []
        for line in out[start+1 :]:
            _, feature, __, cls, weight = filter(lambda _: _, line.split(' '))
            cls = cls[0]
            if cls == '1':
                weight = float(weight)
                feature_weights.append((weight, feature))
        feature_weights.sort(key=lambda (w, _): -w)
        feature_weights = filter(lambda (w, _): 1, feature_weights[:n])
        print 'Value extractor - most informative features:'
        for weight, feature in feature_weights:
            print '%s %s' % (weight, feature)
        print

    def extract_values(self, extracted_sentences, confidence_level=.8):
        setlocale(LC_NUMERIC, 'pl_PL.UTF-8')
        sentences = [
            sentence
            for entity, sentences in extracted_sentences.iteritems()
            for sentence in sentences
        ]
        self.save_features_to_file(self.features_tag_filename, sentences)
        command = 'crfsuite tag -i -m %s %s' % (models_cache_path % self.model_filename, models_cache_path % self.features_tag_filename)
        p = Popen(command, stdout=PIPE, stdin=PIPE, stderr=PIPE, shell=True)
        out, _ = p.communicate()
        tags_list = []
        tags = []
        for line in out.split('\n')[:-1]:
            if not line:
                tags_list.append(tags)
                tags = []
            else:
                tags.append((line[0], float(line[2:])))
        extracted_values = {}
        i = 0
        for entity, sentences in extracted_sentences.iteritems():
            values = []
            for sentence in sentences:
                tags = tags_list[i]
                i += 1
                value = []
                value_prob = 1
                #automatically join hyphenated words
                for j, word in enumerate(sentence):
                    if word.lemma == '-':
                        tags[j] = ('1', 1)
                for word, (tag, p) in zip(sentence, tags) + [('', ('0', 1))]:
                    if tag == '1':
                        if word.lemma == '-' and not value:
                            continue
                        value.append(word.lemma)
                        value_prob = min(value_prob, p)
                    elif value:
                        if value[-1] == '-':
                            value.pop()
                        if not value:
                            continue
                        v = '_'.join(value).replace('_-_', '-')
                        value = []
                        #gmina can have the same name as its main city (in fact, it very often does)
                        if v.decode('utf-8')[:4] != entity.decode('utf-8')[:4] or self.predicate in ['gmina']:
                            values.append((v, value_prob))
                        value_prob = 1
            #sort by decreasing probabilities
            values = filter(lambda (_, p): p > confidence_level, values)
            values.sort(key=lambda (_, p): -p)
            values = map(lambda (v, p): (str(int(atof(v))) if is_numeric(v) else v, p), values)
            values = map(
                lambda (v, p): ((v.decode('utf-8')[0].upper() + v.decode('utf-8')[1:]).encode('utf-8') if '_' in v else v, p), 
                values
            )
            if verbose:
                print entity, values
            values = [v for v, _ in values]
            if values:
                if self.predicate in numeric_predicates:
                    extracted_values[entity] = values[0]
                    continue
                #to increase precision of extraction (at the cost of recall) in textual relations, 
                #only values that are geographic entities in DBPedia are returned
                values_identified_as_entities = [
                    v for v in values if lt.is_entity(v)
                ]
                values_identified_as_entities_of_right_type = [
                    v for v in values_identified_as_entities if\
                    any(entities_types.index(t) in lt.entities[v] for t in self.predominant_types)
                ]
                if verbose:
                    print ' '.join(values),
                    print ' '.join(values_identified_as_entities),
                    print ' '.join(values_identified_as_entities_of_right_type)
                if values_identified_as_entities_of_right_type:
                    extracted_values[entity] = values_identified_as_entities_of_right_type[0]
                elif values_identified_as_entities:
                    extracted_values[entity] = values_identified_as_entities[0]
                elif self.predicate in ['gmina', 'powiat', quote_plus('województwo'), 'hrabstwo', 'stan']:
                    extracted_values[entity] = values[0]
        if not save_to_cache:
            try:
                os.remove(models_cache_path % self.model_filename)
            except IOError:
                pass
        return extracted_values
                   



# -*- coding: utf-8 -*-
import urllib
import urllib2
import itertools
import time
import sys
import json
from collections import defaultdict
from urllib2 import urlopen, Request, unquote

from sparql_access import full_type_name, select_entities_of_type
from config import entities_path
from pickler import Pickler
from config import data_source, sparql_endpoint

entities_types = [
    'Place',
    'PopulatedPlace',
    'Settlement',
    'Country',
    'AdministrativeRegion',
    'Continent',
    'Island',
    'City',
    'River',
    'BodyOfWater',
    'Stream',
    'Lake',
    'NaturalPlace',
    'MountainRange',
    'Valley',
    'Volcano',
    'Mountain'
]

def collect_entities():
    try:
        return Pickler.load(entities_path)
    except IOError:
        pass
    entities = defaultdict(list)
    for i, type in enumerate(entities_types):
        entities_of_type = select_entities_of_type(full_type_name(type))
        for entity in entities_of_type:
            entities[entity].append(i)
            if '_(' in entity:
                entities[entity.split('_(')[0]].append(i)
    Pickler.store(entities, entities_path)
    return entities




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import division
import sys
from itertools import izip
from urllib import quote_plus
from math import ceil

from config import results_path, predicates, evaluation_mode, candidates_limit, verbose, numeric_predicates
from sentence_classifier import get_sentence_classifier, SentenceClassifier
from candidates_selector import CandidatesSelector
from value_extractor import ValueExtractor
from evaluator import run_evaluation
from sparql_access import full_predicate_name, full_resource_name

def learn_new_triples(predicate):
    """learns new triples and stores them in a file"""
    sc = get_sentence_classifier(predicate)
    if predicate in numeric_predicates:
        pattern = '<%s> <%s> "%s"^^<http://www.w3.org/2001/XMLSchema#int> .'
    else:
        pattern = '<%s> <%s> "%s"@pl .'
    entities = CandidatesSelector.get_candidates(predicate)
    entities = entities[: candidates_limit]
    if verbose:
        print '%s candidates identified' % len(entities)
    n = 1000
    entities_list = [
        entities[i*n : (i+1)*n] 
        for i in xrange(int(ceil(len(entities) / n)))
    ]
    out = open(results_path + 'triples-%s' % predicate, 'w')
    ve = ValueExtractor(predicate, sc.extractor_training_data)
    for entities in entities_list:
        extracted_sentences = sc.extract_sentences(entities)
        values = ve.extract_values(extracted_sentences)
        for e, v in values.iteritems():
            if v:
                print >>out, pattern % (
                    full_resource_name(e).encode('utf-8'), 
                    full_predicate_name(predicate).encode('utf-8'), 
                    v
                )

def main():
    for p in predicates:
        if evaluation_mode == 1:
            run_evaluation(p)
        else:
            learn_new_triples(p)
            
if __name__ == '__main__':
    main()




# -*- coding: utf-8 -*-

from __future__ import division
import sys
from itertools import izip
from os.path import join
import pylab as pl

from config import tests_path, verbose
from sentence_classifier import get_sentence_classifier
from candidates_selector import CandidatesSelector
from value_extractor import ValueExtractor
from sparql_access import select_all

class Stats:
    def __init__(self, tp, fp, fn):
        self.tp, self.fp, self.fn = tp, fp, fn
        
    def __repr__(self):
        self.calculate_statistics()
        return 'precision:%.2f recall:%.2f F-measure:%.2f all:%d' % (self.precision, self.recall, self.f_measure, self.tp + self.fn)

    def calculate_statistics(self):
        self.precision = self.tp / max(self.tp + self.fp, 1)
        self.recall = self.tp / max(self.tp + self.fn, 1)
        self.f_measure = 2 * self.precision * self.recall / (self.precision + self.recall) if self.precision != 0 else 0
        
class Evaluator:
    @classmethod
    def evaluate(cls, true_values, values):
        tp, fp, fn = cls.classify_by_error_type(true_values, values)
        s = Stats(len(tp), len(fp), len(fn))
        print s
        return s, fp, fn
    
    @classmethod
    def classify_by_error_type(cls, true_values, suggested_values):
        tp, fp, fn = [], [], []
        for entity, suggested_value in suggested_values.iteritems():
            if not suggested_value:
                continue
            if entity in true_values and cls.value_matches(true_values[entity], suggested_value):
                tp.append(entity)
            else:
                fp.append(entity)
        for entity, true_value in true_values.iteritems():
            if not suggested_values.get(entity) or not cls.value_matches(true_value, suggested_values[entity]):
                fn.append(entity)
        return tp, fp, fn
        
class ValueExtractorEvaluator(Evaluator):
    @staticmethod
    def value_matches(true_values, suggested_value):
        return any(
            v == suggested_value
            for v in true_values    
        )

def get_test_data(predicate):
    entities = open(join(tests_path, predicate, 'entities')).read().split()
    values = open(join(tests_path, predicate, 'values')).read().split('\n')
    true_values = {}
    for value in values:
        value = value.split()
        if value:
            true_values[value[0]] = value[1:]
    return entities, true_values
      
def run_evaluation(predicate, sentence_limit=None):
    entities, true_values = get_test_data(predicate)
    sc = get_sentence_classifier(predicate, sentence_limit)
    true_values = dict((k, v) for k, v in true_values.iteritems() if k in entities)
    if verbose:
        print '%d entities were used in evaluation.' % len(entities)
    extracted_sentences = sc.extract_sentences(entities)
    ve = ValueExtractor(predicate, sc.extractor_training_data)
    values = ve.extract_values(extracted_sentences)
    print '%s results:' % predicate
    stats, fp, fn = ValueExtractorEvaluator.evaluate(true_values, values)
    table_format = '%30s %30s %20s %10s'
    print 'Error table:'
    print table_format % ('Subject:', 'Gold standard values:', 'Extracted value:', 'Error:')
    for entity, value in values.iteritems():
        if entity not in true_values:
            true_values[entity] = '-'
    for entity, true_value in true_values.iteritems():
        if entity in fp and entity in fn:
            err = 'FP/FN'
        elif entity in fp:
            err = 'FP'
        elif entity in fn:
            err = 'FN'
        else:
            err = ''
        print table_format % (entity[:30], ', '.join(true_value), values[entity] if entity in values else '-', err)
    print '\n\n'
    return stats
    
def incremental_evaluation(predicate):
    from collections import defaultdict
    precision = defaultdict(int)
    recall = defaultdict(int)
    n = 10
    x = [10, 25, 50, 100, 250, 500, 1000, 2500]
    for _ in xrange(n):
        for limit in x:
            s = run_evaluation(predicate, limit)
            precision[limit] += s.precision
            recall[limit] += s.recall
    precision = [v/n for _, v in sorted(list(precision.iteritems()))]
    recall = [v/n for _, v in sorted(list(recall.iteritems()))]
    print precision
    print recall    
        



# -*- coding: utf-8 -*-
import sys
import os
import numpy
from itertools import izip
from collections import defaultdict
from random import shuffle
from pprint import pprint
from string import punctuation
from urllib import quote_plus
from urllib2 import unquote

from sklearn.svm import SVC
from sklearn.feature_extraction.text import TfidfVectorizer, CountVectorizer
from sklearn.pipeline import Pipeline

from config import articles_cache_path, models_cache_path, training_limit, verbose, evaluation_mode, numeric_predicates, type_restrictions, save_to_cache
from sparql_access import select_all, select_entities_of_type_in_relation
from pickler import Pickler
from language_tools import LanguageToolsFactory
from article_access import get_article, prepare_articles
from polish_stop_words import stop_words

stop_words = set(stop_words)
lt = LanguageToolsFactory.get_language_tools()
    
def split_camelcase(s):
    ret = []
    word = []
    for letter in s:
        if letter.isupper():
            ret.append(''.join(word))
            word = []
        word.append(letter)
    ret.append(''.join(word))
    return ret
    
def get_sentence_classifier(predicate, sentence_limit=None):
    try:
        ret = Pickler.load(models_cache_path % ('svmmodel-%s.pkl' % predicate))
        ret.classifier.set_params(v__analyzer=lambda x: x)
        return ret
    except IOError:
        return SentenceClassifier(predicate, sentence_limit)
    
class SentenceClassifier:
    def __init__(self, predicate, sentence_limit = None, confidence_level=.7):
        self.predicate = predicate
        self.predicate_words = map(lambda w: w.lower(), split_camelcase(unquote(predicate)))
        self.confidence_level = confidence_level
        self.sentence_limit = sentence_limit
        self.train()
        if save_to_cache:
            #pickle can't save a function, so it's removed before saving
            self.classifier.set_params(v__analyzer=None)
            Pickler.store(self, models_cache_path % ('svmmodel-%s.pkl' % predicate))
            self.classifier.set_params(v__analyzer=lambda x: x)

    def collect_features(self, sentences, threshold=10):
        '''creates a vocabulary of words occurring in the sentences that occur more than threshold times'''
        if self.sentence_limit < 100:
            threshold = 2
        elif self.sentence_limit < 1000:
            threshold = 5
        vocabulary = defaultdict(int)
        for sentence in sentences:
            for feature in self.get_features(sentence):
                vocabulary[feature] += 1
        return [
            feature for feature, count in vocabulary.iteritems() if count > threshold
        ]
        
    def collect_sentences(self, names):
        '''classifies all sentences based on the fact that they contain a reference to the subject of the article, the searched value and if there is more than one such sentence in an article also to at least part of the predicate. Both types of sentences are returned, positive sentences contain also the value.'''
        positive, negative = [], []
        types = ['hrabstwo', 'gmina', 'prowincja', quote_plus('województwo'), 'powiat', 'region']
        for subject, object in names:
            try:
                article = get_article(subject)
            except:
                continue
            pos = []
            object = lt.prepare_value(object, self.predicate)
            best_match = (0, '')
            for sentence in article:
                lemmas = [word.lemma for word in sentence]
                if any(o in lemmas for o in object):
                    if self.predicate not in types or any(p in [l for l in lemmas] for p in self.predicate_words):
                        num_matches = len(set(lemmas) & set(object))
                        if num_matches > best_match[0]:
                            best_match = (num_matches, (sentence, object))
                else:
                    negative.append(sentence)
            if best_match[0]:
                positive.append(best_match[1])
        assert len(positive) > 10, 'Too little training examples.'
        return positive, negative
        
    def train(self):
        if self.predicate in type_restrictions:
            names = select_entities_of_type_in_relation(
                type_restrictions[self.predicate], self.predicate
            )    
        else:
            names = select_all({'p': self.predicate})
        if len(names) > training_limit and self.predicate not in [quote_plus('województwo')]:
            new_names = []
            values_added = set()
            for e, v in names:
                if v not in values_added:
                    values_added.add(v)
                    new_names.append((e, v))
            names = new_names
        names = names[: training_limit]
        if verbose:
            print '%d articles processed during training.' % len(names)
        if evaluation_mode:
            #make sure that entities that will be used in evaluation, are not used in training
            from evaluator import get_test_data
            names = filter(lambda (entity, _): entity not in get_test_data(self.predicate)[0], names)
        #prepare articles about subjects
        prepare_articles(zip(*names)[0])
        positive, negative = self.collect_sentences(names)
        positive = positive[: self.sentence_limit]
        if verbose:
            print 'Sentences selected for training (%d total):' % len(positive)
            for s, v in positive:
                print ' '.join(v), ' ', ' '.join([w.lemma for w in s])
            print
        self.extractor_training_data = positive[:]
        positive = map(lambda (s, v): s, positive)
        #decreases number of negative examples to the number of positive examples to avoid unbalanced data
        shuffle(negative)
        negative = negative[: len(positive)]
        sentences = positive + negative
        classes = [True] * len(positive) + [False] * len(negative)
        vocabulary = self.collect_features(positive)
        if verbose:
            print 'Words considered as features:'
            print vocabulary
            print
        self.classifier = Pipeline([
            ('v', CountVectorizer(analyzer=lambda x: x, vocabulary=vocabulary, binary=True)),
            ('c', SVC(kernel='linear', probability=True)),
        ]) 
        self.classifier.fit(map(self.get_features, sentences), classes)
        self.get_most_informative_features()
        
    def get_most_informative_features(self, n=10):
        #works only with a linear kernel
        try:
            vectorizer = self.classifier.named_steps['v']
            clf = self.classifier.named_steps['c']
            feature_relevance = list(reversed(sorted(zip(clf.coef_[0].toarray()[0], vectorizer.get_feature_names()))))[:n]
            #filter out nondiscriminating features
            feature_relevance = filter(lambda (v, _): v >= .5, feature_relevance)
            if verbose:
                print 'Most informative features:'
                for value, name in feature_relevance:
                    print '%s %.2f' % (name, value)
                print
        except ValueError:
            pass
        
    def extract_sentences(self, entities):
        articles = prepare_articles(entities)
        extracted_sentences = defaultdict(list)
        if verbose:
            print 'Classifying sentences:'
        for entity in entities:
            try:
                article = get_article(entity)
            except:
                continue
            if not article:
                continue
            if verbose:
                print entity
            probabilities = [prob[1] for prob in self.classifier.predict_proba(map(self.get_features, article))]
            #for each article return all sentences with scores > confidence_level
            for sentence, p in izip(article, probabilities):
                if p > self.confidence_level:
                    extracted_sentences[entity].append(sentence)
                    if verbose:
                        print '***', '%.2f' % p, ' '.join([w.segment for w in sentence])
                elif verbose:
                    print '%.2f' % p, ' '.join([w.segment for w in sentence])
            if verbose:
                print
        return extracted_sentences
        
    def get_features(self, sentence):
        return filter(
            lambda w: w.decode('utf-8')[0].islower() and w not in stop_words, 
            [word.lemma for word in sentence]
        )
       



#encoding: utf-8
import sys
from collections import defaultdict

from config import candidates_cache_path, type_restrictions
from sparql_access import select_types, count_entities_of_type, select_entities_of_type_not_in_relation, full_type_name
from pickler import Pickler

class CandidatesSelector:
    #types that are too wide, cover too many entities and should never be returned
    wide_types = [
        'http://schema.org/Place',
        'http://www.w3.org/2002/07/owl#Thing',
        'http://dbpedia.org/ontology/Place',
        'http://dbpedia.org/ontology/NaturalPlace',
        'http://dbpedia.org/ontology/BodyOfWater',
        'http://www.opengis.net/gml/_Feature'
    ]

    @staticmethod
    def get_predominant_types(predicate, subject=True):
        if predicate in type_restrictions:
            return [full_type_name(type_restrictions[predicate])]
        #limiting these relations to settlements only increases precision a lot
        if predicate in ['stolica', 'gmina', 'region', 'prowincja', 'hrabstwo']:
            return [u'http://dbpedia.org/ontology/Settlement']
        type_preciseness = .75
        types_list = select_types(predicate, subject)
        types_count = defaultdict(int)
        for types in types_list:
            for type in types:
                if type not in CandidatesSelector.wide_types:
                    types_count[type] += 1
        num_entities = len(types_list)
        return [
            type for type, count in types_count.iteritems()
            if count >= type_preciseness * num_entities
        ]
        
    @staticmethod
    def get_most_specific_types(predominant_types):
        threshold = 100
        return [
            type for type in predominant_types
            if count_entities_of_type(type) > threshold
        ]
    
    @staticmethod
    def get_candidates(predicate):
        try:
            return Pickler.load(candidates_cache_path % predicate)
        except IOError:
            pass
        types = CandidatesSelector.get_most_specific_types(
            CandidatesSelector.get_predominant_types(predicate)
        )
        if types: 
            candidates = select_entities_of_type_not_in_relation(
                types[0], 
                predicate
            )
            if predicate == 'gmina':
                candidates = filter(lambda e: 'Gmina' not in e, candidates)
            if predicate == 'powiat':
                candidates = filter(lambda e: 'Powiat' not in e, candidates)
            if predicate == 'hrabstwo':
                candidates = filter(lambda e: 'hrabstwo_miejskie' not in e and 'Hrabstwo' not in e, candidates)
            Pickler.store(candidates, candidates_cache_path % predicate)
            return candidates
        else:
            return []




#encoding: utf-8
import sys
import os
import glob
from os.path import join
from codecs import open as copen

from wikipedia_cleaner import clean
from config import raw_articles_path, articles_cache_path
from language_tools import LanguageToolsFactory, article_sentence_limit
from pickler import Pickler

lt = LanguageToolsFactory.get_language_tools()

class ArticleNotFoundError(BaseException):
    pass

def get_raw_article(name):
    try:
        f = copen(join(raw_articles_path, name), encoding='utf-8')
        return clean(f.read())[: article_sentence_limit]
    except IOError:
        raise ArticleNotFoundError(name)

def get_article(name):
    try:
        return Pickler.load(articles_cache_path % name)[: article_sentence_limit]
    except IOError:
        raise ArticleNotFoundError(name)
        
def prepare_articles(names):
    '''saves tagged articles about given entities in a cache'''
    for f in glob.glob(join(raw_articles_path, "*.txt*")):
        os.remove(f)
    found = False
    link_dictionaries = {}
    for i, name in enumerate(names):
        try:
            get_article(name)
        except ArticleNotFoundError:
            try:
                article, link_dictionary = get_raw_article(name)
                link_dictionaries[i] = link_dictionary
            except ArticleNotFoundError:
                continue
            found = True
            article = '\n'.join(article.split('\n')[: article_sentence_limit])
            out = copen(join(raw_articles_path, '%d.txt' % i), 'w', 'utf-8')
            print >>out, article
    if found:
        articles = lt.run_nlptools(link_dictionaries)
        for f in glob.glob(join(raw_articles_path, "*.txt*")):
            os.remove(f)
        #save processed articles
        for i, article in articles.iteritems():
            Pickler.store(article, articles_cache_path % names[i])




#!/usr/bin/python
# -*- coding: utf-8 -*-

# =============================================================================
#  Version: 1.5 (Oct 17, 2009)
#  Author: Antonio Fuschetto (fuschett@di.unipi.it), University of Pisa
# =============================================================================

# =============================================================================
#  This file is part of Tanl.
#
#  Tanl is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License, version 3,
#  as published by the Free Software Foundation.
#
#  Tanl is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# =============================================================================

# Changes by Marcin Zając signified by #MZ

"""Wikipedia Extractor:
Extracts and cleans text from Wikipedia database dump and stores output in a
number of files of similar size in a given directory. Each file contains
several documents in Tanl document format.

Usage:
  WikiExtractor.py [options]

Options:
  -c, --compress        : compress output files using bzip2 algorithm
  -b ..., --bytes=...   : put specified bytes per output file (500K by default)
  -o ..., --output=...  : place output files in specified directory (current
                          directory by default)
  --help                : display this help and exit
  --usage               : display script usage
"""

import sys
import getopt
import pickle
import urllib
import re
import bz2
import os.path
from urllib2 import unquote

### PARAMS ####################################################################

prefix = 'http://it.wikipedia.org/wiki/'

### SUPPORT CLASSES ###########################################################


link_dictionary = {}
from config import entities_path
from pickler import Pickler
from collect_entities import collect_entities
entities = collect_entities()

class WikiDocument:
    def __init__(self):
        self.id = None
        self.url = None
        self.text = None

    def __str__(self):
        return '<doc id="%d" url="%s">\n%s\n</doc>\n' % (self.id, self.url, self.text)

def get_wiki_document_url(wiki_document_title, prefix):
    quoted_title = urllib.quote(wiki_document_title.replace(' ', '_').encode('utf-8'))
    quoted_title = quoted_title.replace('%28', '(').replace('%29', ')')
    return prefix + (quoted_title[0].upper() if quoted_title else '') + quoted_title[1:]

#------------------------------------------------------------------------------

class WikiExtractor:
    __garbage_tags = ('ref', 'gallery', 'timeline', 'noinclude', 'pre', 'table', 'tr', 'td',
                      'ul', 'li', 'ol', 'dl', 'dt', 'dd', 'menu', 'dir')
    __wrapper_tags = ('nowiki', 'cite', 'source', 'hiero', 'div', 'font', 'span', 'strong',
                      'strike', 'blockquote', 'tt', 'var', 'sup', 'sub', 'big', 'small',
                      'center', 'h1', 'h2', 'h3', 'em', 'b', 'i', 'u', 'a', 's', 'p')
    __single_tags = ('references', 'ref', 'img', 'br', 'hr', 'li', 'dt', 'dd')
    __placeholder_tags = {'math':'formula', 'code':'codice'}

    __project_namespaces = ('wikipedia', 'mediawiki', 'wikiquote', 'wikibooks', 'wikisource',
                            'wiktionary', 'wikispecies', 'wikinews', 'wikiversita', 'commons')
    __garbage_namespaces = ('immagine', 'image', 'categoria', 'category', 'file')

    __char_entities =  {'&nbsp;'   :u'\u00A0', '&iexcl;' :u'\u00A1', '&cent;'    :u'\u00A2',
                        '&pound;'  :u'\u00A3', '&curren;':u'\u00A4', '&yen;'     :u'\u00A5',
                        '&brvbar;' :u'\u00A6', '&sect;'  :u'\u00A7', '&uml;'     :u'\u00A8',
                        '&copy;'   :u'\u00A9', '&ordf;'  :u'\u00AA', '&laquo;'   :u'\u00AB',
                        '&not;'    :u'\u00AC', '&shy;'   :u'\u00AD', '&reg;'     :u'\u00AE',
                        '&macr;'   :u'\u00AF', '&deg;'   :u'\u00B0', '&plusmn;'  :u'\u00B1',
                        '&sup2;'   :u'\u00B2', '&sup3;'  :u'\u00B3', '&acute;'   :u'\u00B4',
                        '&micro;'  :u'\u00B5', '&para;'  :u'\u00B6', '&middot;'  :u'\u00B7',
                        '&cedil;'  :u'\u00B8', '&sup1;'  :u'\u00B9', '&ordm;'    :u'\u00BA',
                        '&raquo;'  :u'\u00BB', '&frac14;':u'\u00BC', '&frac12;'  :u'\u00BD',
                        '&frac34;' :u'\u00BE', '&iquest;':u'\u00BF', '&Agrave;'  :u'\u00C0',
                        '&Aacute;' :u'\u00C1', '&Acirc;' :u'\u00C2', '&Atilde;'  :u'\u00C3',
                        '&Auml;'   :u'\u00C4', '&Aring;' :u'\u00C5', '&AElig;'   :u'\u00C6',
                        '&Ccedil;' :u'\u00C7', '&Egrave;':u'\u00C8', '&Eacute;'  :u'\u00C9',
                        '&Ecirc;'  :u'\u00CA', '&Euml;'  :u'\u00CB', '&Igrave;'  :u'\u00CC',
                        '&Iacute;' :u'\u00CD', '&Icirc;' :u'\u00CE', '&Iuml;'    :u'\u00CF',
                        '&ETH;'    :u'\u00D0', '&Ntilde;':u'\u00D1', '&Ograve;'  :u'\u00D2',
                        '&Oacute;' :u'\u00D3', '&Ocirc;' :u'\u00D4', '&Otilde;'  :u'\u00D5',
                        '&Ouml;'   :u'\u00D6', '&times;' :u'\u00D7', '&Oslash;'  :u'\u00D8',
                        '&Ugrave;' :u'\u00D9', '&Uacute;':u'\u00DA', '&Ucirc;'   :u'\u00DB',
                        '&Uuml;'   :u'\u00DC', '&Yacute;':u'\u00DD', '&THORN;'   :u'\u00DE',
                        '&szlig;'  :u'\u00DF', '&agrave;':u'\u00E0', '&aacute;'  :u'\u00E1',
                        '&acirc;'  :u'\u00E2', '&atilde;':u'\u00E3', '&auml;'    :u'\u00E4',
                        '&aring;'  :u'\u00E5', '&aelig;' :u'\u00E6', '&ccedil;'  :u'\u00E7',
                        '&egrave;' :u'\u00E8', '&eacute;':u'\u00E9', '&ecirc;'   :u'\u00EA',
                        '&euml;'   :u'\u00EB', '&igrave;':u'\u00EC', '&iacute;'  :u'\u00ED',
                        '&icirc;'  :u'\u00EE', '&iuml;'  :u'\u00EF', '&eth;'     :u'\u00F0',
                        '&ntilde;' :u'\u00F1', '&ograve;':u'\u00F2', '&oacute;'  :u'\u00F3',
                        '&ocirc;'  :u'\u00F4', '&otilde;':u'\u00F5', '&ouml;'    :u'\u00F6',
                        '&divide;' :u'\u00F7', '&oslash;':u'\u00F8', '&ugrave;'  :u'\u00F9',
                        '&uacute;' :u'\u00FA', '&ucirc;' :u'\u00FB', '&uuml;'    :u'\u00FC',
                        '&yacute;' :u'\u00FD', '&thorn;' :u'\u00FE', '&yuml;'    :u'\u00FF',
                        '&fnof;'   :u'\u0192', '&Alpha;' :u'\u0391', '&Beta;'    :u'\u0392',
                        '&Gamma;'  :u'\u0393', '&Delta;' :u'\u0394', '&Epsilon;' :u'\u0395',
                        '&Zeta;'   :u'\u0396', '&Eta;'   :u'\u0397', '&Theta;'   :u'\u0398',
                        '&Iota;'   :u'\u0399', '&Kappa;' :u'\u039A', '&Lambda;'  :u'\u039B',
                        '&Mu;'     :u'\u039C', '&Nu;'    :u'\u039D', '&Xi;'      :u'\u039E',
                        '&Omicron;':u'\u039F', '&Pi;'    :u'\u03A0', '&Rho;'     :u'\u03A1',
                        '&Sigma;'  :u'\u03A3', '&Tau;'   :u'\u03A4', '&Upsilon;' :u'\u03A5',
                        '&Phi;'    :u'\u03A6', '&Chi;'   :u'\u03A7', '&Psi;'     :u'\u03A8',
                        '&Omega;'  :u'\u03A9', '&alpha;' :u'\u03B1', '&beta;'    :u'\u03B2',
                        '&gamma;'  :u'\u03B3', '&delta;' :u'\u03B4', '&epsilon;' :u'\u03B5',
                        '&zeta;'   :u'\u03B6', '&eta;'   :u'\u03B7', '&theta;'   :u'\u03B8',
                        '&iota;'   :u'\u03B9', '&kappa;' :u'\u03BA', '&lambda;'  :u'\u03BB',
                        '&mu;'     :u'\u03BC', '&nu;'    :u'\u03BD', '&xi;'      :u'\u03BE',
                        '&omicron;':u'\u03BF', '&pi;'    :u'\u03C0', '&rho;'     :u'\u03C1',
                        '&sigmaf;' :u'\u03C2', '&sigma;' :u'\u03C3', '&tau;'     :u'\u03C4',
                        '&upsilon;':u'\u03C5', '&phi;'   :u'\u03C6', '&chi;'     :u'\u03C7',
                        '&psi;'    :u'\u03C8', '&omega;' :u'\u03C9', '&thetasym;':u'\u03D1',
                        '&upsih;'  :u'\u03D2', '&piv;'   :u'\u03D6', '&bull;'    :u'\u2022',
                        '&hellip;' :u'\u2026', '&prime;' :u'\u2032', '&Prime;'   :u'\u2033',
                        '&oline;'  :u'\u203E', '&frasl;' :u'\u2044', '&weierp;'  :u'\u2118',
                        '&image;'  :u'\u2111', '&real;'  :u'\u211C', '&trade;'   :u'\u2122',
                        '&alefsym;':u'\u2135', '&larr;'  :u'\u2190', '&uarr;'    :u'\u2191',
                        '&rarr;'   :u'\u2192', '&darr;'  :u'\u2193', '&harr;'    :u'\u2194',
                        '&crarr;'  :u'\u21B5', '&lArr;'  :u'\u21D0', '&uArr;'    :u'\u21D1',
                        '&rArr;'   :u'\u21D2', '&dArr;'  :u'\u21D3', '&hArr;'    :u'\u21D4',
                        '&forall;' :u'\u2200', '&part;'  :u'\u2202', '&exist;'   :u'\u2203',
                        '&empty;'  :u'\u2205', '&nabla;' :u'\u2207', '&isin;'    :u'\u2208',
                        '&notin;'  :u'\u2209', '&ni;'    :u'\u220B', '&prod;'    :u'\u220F',
                        '&sum;'    :u'\u2211', '&minus;' :u'\u2212', '&lowast;'  :u'\u2217',
                        '&radic;'  :u'\u221A', '&prop;'  :u'\u221D', '&infin;'   :u'\u221E',
                        '&ang;'    :u'\u2220', '&and;'   :u'\u2227', '&or;'      :u'\u2228',
                        '&cap;'    :u'\u2229', '&cup;'   :u'\u222A', '&int;'     :u'\u222B',
                        '&there4;' :u'\u2234', '&sim;'   :u'\u223C', '&cong;'    :u'\u2245',
                        '&asymp;'  :u'\u2248', '&ne;'    :u'\u2260', '&equiv;'   :u'\u2261',
                        '&le;'     :u'\u2264', '&ge;'    :u'\u2265', '&sub;'     :u'\u2282',
                        '&sup;'    :u'\u2283', '&nsub;'  :u'\u2284', '&sube;'    :u'\u2286',
                        '&supe;'   :u'\u2287', '&oplus;' :u'\u2295', '&otimes;'  :u'\u2297',
                        '&perp;'   :u'\u22A5', '&sdot;'  :u'\u22C5', '&lceil;'   :u'\u2308',
                        '&rceil;'  :u'\u2309', '&lfloor;':u'\u230A', '&rfloor;'  :u'\u230B',
                        '&lang;'   :u'\u2329', '&rang;'  :u'\u232A', '&loz;'     :u'\u25CA',
                        '&spades;' :u'\u2660', '&clubs;' :u'\u2663', '&hearts;'  :u'\u2665',
                        '&diams;'  :u'\u2666', '&quot;'  :u'\u0022', '&lt;'      :u'\u003C',
                        '&gt;'     :u'\u003E', '&OElig;' :u'\u0152', '&oelig;'   :u'\u0153',
                        '&Scaron;' :u'\u0160', '&scaron;':u'\u0161', '&Yuml;'    :u'\u0178',
                        '&circ;'   :u'\u02C6', '&tilde;' :u'\u02DC', '&ensp;'    :u'\u2002',
                        '&emsp;'   :u'\u2003', '&thinsp;':u'\u2009', '&zwnj;'    :u'\u200C',
                        '&zwj;'    :u'\u200D', '&lrm;'   :u'\u200E', '&rlm;'     :u'\u200F',
                        '&ndash;'  :u'\u2013', '&mdash;' :u'\u2014', '&lsquo;'   :u'\u2018',
                        '&rsquo;'  :u'\u2019', '&sbquo;' :u'\u201A', '&ldquo;'   :u'\u201C',
                        '&rdquo;'  :u'\u201D', '&bdquo;' :u'\u201E', '&dagger;'  :u'\u2020',
                        '&Dagger;' :u'\u2021', '&permil;':u'\u2030', '&lsaquo;'  :u'\u2039',
                        '&rsaquo;' :u'\u203A', '&euro;'  :u'\u20AC'}

    def __init__(self):
        # Riconosce i commenti HTML
        self.__comment_pattern = re.compile(r'<!--.*?-->', re.DOTALL)

        # Riconosce i tag HTML spazzatura
        self.__garbage_tag_patterns = list()
        for tag in self.__class__.__garbage_tags:
            pattern = re.compile(r'<\s*%s(\s*| [^/]+?)>.*?<\s*/\s*%s\s*>' % (tag, tag), re.DOTALL | re.IGNORECASE)
            self.__garbage_tag_patterns.append(pattern)

        # Riconosce i tag HTML contenitori
        self.__wrapper_tag_patterns = list()
        for tag in self.__class__.__wrapper_tags:
            left_pattern = re.compile(r'<\s*%s(\s*| [^/]+?)>' % tag, re.DOTALL | re.IGNORECASE)
            right_pattern = re.compile(r'<\s*/\s*%s\s*>' % tag, re.DOTALL | re.IGNORECASE)
            self.__wrapper_tag_patterns.append((left_pattern, right_pattern))

        # Riconosce i tag HTML singoli
        self.__single_tag_patterns = list()
        for tag in self.__class__.__single_tags:
            good_pattern = re.compile(r'<\s*%s(\s*| .+?)/\s*>' % tag, re.DOTALL | re.IGNORECASE)
            bad_pattern = re.compile(r'<\s*(/|\\)?\s*%s(\s*| [^/]+?)\\?\s*>' % tag, re.DOTALL | re.IGNORECASE)
            self.__single_tag_patterns.append((good_pattern, bad_pattern))

        # Riconosce i tag HTML segnaposto
        self.__placeholder_tag_patterns = list()
        for tag in self.__class__.__placeholder_tags.iterkeys():
            pattern = re.compile(r'<\s*%s(\s*| [^/]+?)>.*?<\s*/\s*%s\s*>' % (tag, tag), re.DOTALL | re.IGNORECASE)
            self.__placeholder_tag_patterns.append((pattern, self.__class__.__placeholder_tags[tag]))

        # Riconosce le tabelle e i template
        self.__table_pattern = re.compile(r'\{[^{]*?\}', re.DOTALL)

        # Riconosce i wikilink
        good_wikilink_pattern = re.compile(r'\[\[[^[]*?\]\]', re.DOTALL)
        bad_left_wikilink_pattern = re.compile(r'\[[^[]*?\]\]', re.DOTALL)
        bad_right_wikilink_pattern = re.compile(r'\[\[[^[]*?\]', re.DOTALL)
        self.__wikilink_pattern = (good_wikilink_pattern, bad_left_wikilink_pattern, bad_right_wikilink_pattern)

        # Riconosce i link HTTP
        self.__http_link_pattern = re.compile(r'\[http.*?\]', re.DOTALL | re.IGNORECASE)

        # Riconosce gli apostrofi che precedono grassetto e corsivo
        apostrophe_bold_pattern = re.compile(r"\w'('''[^\s'][^']*?[^\s']''')[^']", re.DOTALL)
        apostrophe_italic_pattern = re.compile(r"\w'(''[^\s'][^']*?[^\s']'')[^']", re.DOTALL)
        self.__apostrophe_pattern = (apostrophe_bold_pattern, apostrophe_italic_pattern)

        # Riconosce le entita' numeriche
        self.__numeric_entity_pattern = re.compile(r'&#\d+?;')

        # Riconosce gli spazi multipli
        self.__multi_space_pattern = re.compile(r' {2,}')

        # Riconosce i punti multipli
        self.__multi_dot_pattern = re.compile(r'\.{4,}')

    def extract(self, wiki_document):
        wiki_document = self.__clean(wiki_document)
        if not wiki_document: return None

        wiki_document = self.__compact(wiki_document)
        return wiki_document

    def __clean(self, wiki_document):
        #MZ, treat the article title as a link
        wiki_document.text = wiki_document.text.replace("'''", '[[', 1).replace("'''", ']]', 1)
        # Rende maggiormente riconoscibili i tag
        wiki_document.text = wiki_document.text.replace('&lt;', '<').replace('&gt;', '>')
        wiki_document.text = wiki_document.text.replace('<<', u'«').replace('>>', u'»')

        # Elimina i commenti HTML
        wiki_document.text = self.__comment_pattern.sub('', wiki_document.text)

        # Elimina i tag HTML spazzatura
        for pattern in self.__garbage_tag_patterns:
            wiki_document.text = pattern.sub('', wiki_document.text)

        # Elimina i tag HTML contenitori
        for left_pattern, right_pattern in self.__wrapper_tag_patterns:
            wiki_document.text = left_pattern.sub('', wiki_document.text)
            wiki_document.text = right_pattern.sub('', wiki_document.text)

        # Elimina i tag HTML singoli
        for good_pattern, bad_pattern in self.__single_tag_patterns:
            wiki_document.text = good_pattern.sub('', wiki_document.text)
            wiki_document.text = bad_pattern.sub('', wiki_document.text)

        # Elimina i tag HTML segnaposto
        for pattern, placeholder in self.__placeholder_tag_patterns:
            index = 1
            for match in pattern.finditer(wiki_document.text):
                wiki_document.text = wiki_document.text.replace(match.group(), '%s_%d' % (placeholder, index))
                index += 1

        # Elimina le tabelle e i template
        wiki_document.text = wiki_document.text.replace('{{end box}}', '}')
        wiki_document.text = wiki_document.text.replace('{{', '{').replace('}}', '}')
        wiki_document.text = wiki_document.text.replace('{|', '{').replace('|}', '}')
        wiki_document.text = self.__table_pattern.sub('', wiki_document.text)
        wiki_document.text = self.__table_pattern.sub('', wiki_document.text)
        wiki_document.text = self.__table_pattern.sub('', wiki_document.text)

        # Gestisce i wikilink (ben formattati; due livelli di annidamento)
        good_wikilink_pattern = self.__wikilink_pattern[0]
        for match in good_wikilink_pattern.finditer(wiki_document.text):
            wikilink = match.group()
            document_title, link_text = self.__handle_wikilink(wikilink[2:-2])
            wiki_document.text = wiki_document.text.replace(wikilink, self.__get_anchor_tag(document_title, link_text))
        for match in good_wikilink_pattern.finditer(wiki_document.text):
            wikilink = match.group()
            wiki_document.text = wiki_document.text.replace(wikilink, self.__handle_wikilink(wikilink[2:-2])[1])

        # Gestisce i wikilink (mal formattati)
        bad_left_wikilink_pattern = self.__wikilink_pattern[1]
        for match in bad_left_wikilink_pattern.finditer(wiki_document.text):
            wikilink = match.group()
            document_title, link_text = self.__handle_wikilink(wikilink[1:-2])
            wiki_document.text = wiki_document.text.replace(wikilink, self.__get_anchor_tag(document_title, link_text))
        bad_right_wikilink_pattern = self.__wikilink_pattern[2]
        for match in bad_right_wikilink_pattern.finditer(wiki_document.text):
            wikilink = match.group()
            document_title, link_text = self.__handle_wikilink(wikilink[2:-1])
            wiki_document.text = wiki_document.text.replace(wikilink, self.__get_anchor_tag(document_title, link_text))
        wiki_document.text = wiki_document.text.replace('[[', '').replace(']]', '')

        # Elimina i link HTTP
        wiki_document.text = self.__http_link_pattern.sub('', wiki_document.text).replace('[]', '')

        # Gestisce i grassetti e i corsivi
        apostrophe_bold_pattern = self.__apostrophe_pattern[0]
        for match in apostrophe_bold_pattern.finditer(wiki_document.text):
            bold_text = match.group(1)
            wiki_document.text = wiki_document.text.replace(bold_text, bold_text[3:-3])
        apostrophe_italic_pattern = self.__apostrophe_pattern[1]
        for match in apostrophe_italic_pattern.finditer(wiki_document.text):
            italic_text = match.group(1)
            wiki_document.text = wiki_document.text.replace(italic_text, '&quot;%s&quot;' % italic_text[2:-2])
        wiki_document.text = wiki_document.text.replace("'''", '').replace("''", '&quot;')

        # Gestisce i caratteri speciali
        wiki_document.text = wiki_document.text.replace('&amp;', '&').replace('&quot;&quot;', '&quot;')
        for entity in self.__class__.__char_entities.iterkeys():
            wiki_document.text = wiki_document.text.replace(entity, self.__class__.__char_entities[entity])

        # Gestisce i caratteri speciali
        for match in self.__numeric_entity_pattern.finditer(wiki_document.text):
            entity = match.group()
            wiki_document.text = wiki_document.text.replace(entity, self.__handle_unicode(entity))

        # Gestisce alcune imperfezioni del testo
        wiki_document.text = wiki_document.text.replace('\t', ' ')
        wiki_document.text = self.__multi_space_pattern.sub(' ', wiki_document.text)
        wiki_document.text = self.__multi_dot_pattern.sub('...', wiki_document.text)
        wiki_document.text = wiki_document.text.replace(' ,', ',').replace(' .', '.')
        wiki_document.text = wiki_document.text.replace(' :', ':').replace(' ;', ';')
        wiki_document.text = wiki_document.text.replace(',,', ',').replace(',.', '.')
        wiki_document.text = wiki_document.text.replace('( ', '(').replace(' )', ')')
        wiki_document.text = wiki_document.text.replace('[ ', '[').replace(' ]', ']')
        wiki_document.text = wiki_document.text.replace(u'« ', u'«').replace(u' »', u'»')

        wiki_document.link_dictionary = link_dictionary
        global link_dictionary
        link_dictionary = {}
        return wiki_document

    def __compact(self, wiki_document):
        #MZ: dont collect paragraph names
        page = list()
        paragraph = list()
        for line in wiki_document.text.split('\n'):
            line = line.strip()
            if not line: continue

            # Gestisce il titolo della pagina
            if line.startswith('++'):
                continue #MZ
                title = line[2:-2]
                if title and title[-1] not in '!?':
                    title = '%s.' % title
                page = [title]
            # Gestisce i titoli dei paragrafi
            elif line.startswith('=='):
                continue #MZ
                if len(paragraph) > 1:
                    page.extend(paragraph)
                title = line[2:-2]
                if title and title[-1] not in '!?':
                    title = '%s.' % title
                paragraph = [title]
            # Elimina gli elenchi puntati e numerati
            elif line[-1] == ':' or line[0] in '*#:;':
                continue
            # Elimina i resti delle tabelle
            elif line[0] in '{|' or line[-1] in '}':
                continue
            # Elimina le righe non significative
            elif (line[0] == '(' and line[-1] == ')') or line.strip('.-') == '':
                continue
            # Elimina le righe con un basso numero di token
            elif not '_' in line and len(line.split()) < 6:
                continue
            # Gestisce il testo della pagina
            elif len(paragraph) == 0:
                page.append(line)
            # Gestisce il testo dei paragrafi
            else:
                paragraph.append(line)

        if len(paragraph) > 1:
            page.extend(paragraph)
        #MZ        
#        elif len(page) == 1: return None

        wiki_document.text = '\n'.join(page)
        return wiki_document

    def __handle_wikilink(self, wikilink):
        tokens = wikilink.split(':')
        while not tokens[0]:
            if len(tokens) < 2: return '', ''
            tokens = tokens[1:]

        if len(tokens) == 1 or tokens[0].strip().lower() in self.__class__.__project_namespaces:
            tokens = tokens[-1].split('|')
            while not tokens[-1]:
                if len(tokens) < 2: return '', ''
                tokens = tokens[:-1]
            link_text = tokens[-1].split('#')[-1].split('/')[-1].strip()
            if len(tokens) > 1:
                article_title = tokens[-2].strip()
            else:
                 article_title = link_text
            return article_title, link_text

        if tokens[0].strip().lower() in self.__class__.__garbage_namespaces: return '', ''

        tokens = tokens[-1].split('|')
        while not tokens[-1]:
            if len(tokens) < 2: return '', ''
            tokens = tokens[:-1]
        if len(tokens) == 1: return '', ''
        link_text = tokens[-1].split('#')[-1].split('/')[-1].strip()
        if len(tokens) > 1:
            article_title = tokens[-2].strip()
        else:
            article_title = link_text
        return article_title, link_text

    def __get_anchor_tag(self, document_title, link_text):
        def longer_common_prefix(s1, s2, pattern):
            for i in xrange(min([len(s1), len(s2), len(pattern)])):
                if s2[i] != pattern[i]:
                    return s1
                if s1[i] != pattern[i]:
                    return s2
            return s1 if len(s1) >= len(s2) else s2
    
        if not link_text:
            return ''
        link_text = link_text.replace(' ', '_')
        link_name = unquote(get_wiki_document_url(document_title, ''))
        if link_name in entities:
            ts = link_text.split('_')
            for t in ts:
                if '-' in t:
                    ts += t.split('-')
            for t in ts:
                ns = link_name.decode('utf-8').split('_')
                for n in ns:
                    if '-' in n:
                        ns += n.split('-')
                for n in ns:
                    if t.lower()[:2] == n.lower()[:2]:
                        if t in link_dictionary:
                            link_dictionary[t] = longer_common_prefix(link_dictionary[t], n, t)                        
                        else:
                            link_dictionary[t] = n
        return link_text.replace('_', ' ')

    def __handle_unicode(self, entity):
        numeric_code = int(entity[2:-1])
        if numeric_code >= 0x10000: return ''
        return unichr(numeric_code)

#------------------------------------------------------------------------------

class OutputSplitter:
    def __init__(self, compress, max_file_size, path_name):
        self.__dir_index = 0
        self.__file_index = -1
        self.__cur_file_size = 0
        self.__compress = compress
        self.__max_file_size = max_file_size
        self.__path_name = path_name
        self.__out_file = self.__open_next_file()

    def write(self, text):
        text_len = len(text)
        if self.__cur_file_size + text_len / 2 > self.__max_file_size:
            self.__close_cur_file()
            self.__out_file = self.__open_next_file()
            self.__cur_file_size = 0
        self.__out_file.write(text)
        self.__cur_file_size += text_len

    def close(self):
        self.__close_cur_file()

    def __open_next_file(self):
        self.__file_index += 1
        if self.__file_index == 100:
            self.__dir_index += 1
            self.__file_index = 0
        dir_name = self.__get_dir_name()
        if not os.path.isdir(dir_name):
            os.makedirs(dir_name)
        file_name = os.path.join(dir_name, self.__get_file_name())
        if self.__compress:
            return bz2.BZ2File('%s.bz2' % file_name, 'w')
        else:
            return open(file_name, 'w')

    def __close_cur_file(self):
        self.__out_file.close()

    def __get_dir_name(self):
        char1 = self.__dir_index % 26
        char2 = self.__dir_index / 26 % 26
        return os.path.join(self.__path_name, '%c%c' % (ord('A') + char2, ord('A') + char1))

    def __get_file_name(self):
        return 'wiki%02d' % self.__file_index

### CORE ######################################################################

def process_data(input_file, wiki_extractor, output_splitter):
    page = []
    for line in input_file:
        line = line.decode('utf-8').strip()
        if line == '<page>':
            page = []
        elif line == '</page>':
            process_page(page, wiki_extractor, output_splitter)
        else:
            page.append(line)

#------------------------------------------------------------------------------

def process_page(page, wiki_extractor, output_splitter):
    wiki_document = extract_document(page)
    if not wiki_document: return

    wiki_document = wiki_extractor.extract(wiki_document)
    if not wiki_document: return

    output_splitter.write(wiki_document.__str__().encode('utf-8'))

#------------------------------------------------------------------------------

def extract_document(page):
    wiki_document = WikiDocument()
    for line in page:
        if not line: continue

        # Identificatore della pagina (nodo XML)
        if not wiki_document.id and line.startswith('<id>') and line.endswith('</id>'):
            wiki_document.id = int(line[4:-5])
            continue
        # Titolo della pagina (nodo XML)
        elif not wiki_document.url and line.startswith('<title>') and line.endswith('</title>'):
            title = line[7:-8].replace('&amp;', '&')
            if ':' in title: return None
            wiki_document.url = get_wiki_document_url(title, prefix)
            wiki_document.text = '++%s++' % title
            continue
        # Inizio del testo della pagina (nodo XML)
        elif line.startswith('<text'):
            if line.endswith('</text>'): return None
            line = line[27:]
            if not line: continue
        # Fine del testo della pagina (nodo XML)
        elif line.endswith('</text>'):
            line = line[:-7]
            if not line: continue
        # Informazione superflua (nodo XML)
        elif line[0] == '<':
            continue
        # Titolo di paragafo (testo della pagina)
        elif line[0] == '=':
            line = '==%s==' % line.strip('= ')

        wiki_document.text += '\n%s' % line
    return wiki_document

### USER INTERFACE ############################################################

def show_help():
    print >> sys.stdout, __doc__,

def show_usage(output_file, script_name):
    print >> output_file, 'Usage: %s [options]' % script_name

def show_suggestion(output_file, script_name):
    print >> output_file, 'Try \'%s --help\' for more information.' % script_name

def show_size_error(script_name, file_size):
    print >> sys.stderr, '%s: %s: Insufficient or invalid number of bytes' % (script_name, file_size)

def show_file_error(script_name, file_name):
    print >> sys.stderr, '%s: %s: No such file or directory' % (script_name, file_name)

def main():
    script_name = os.path.basename(sys.argv[0])

    try:
        long_opts = ['help', 'usage', 'compress', 'bytes=', 'output=']
        opts, args = getopt.gnu_getopt(sys.argv[1:], 'cb:o:', long_opts)
    except getopt.GetoptError:
        show_usage(sys.stderr, script_name)
        show_suggestion(sys.stderr, script_name)
        sys.exit(1)

    compress = False
    file_size = 500 * 1024
    output_dir = '.'

    for opt, arg in opts:
        if opt == '--help':
            show_help()
            sys.exit()
        elif opt == '--usage':
            show_usage(sys.stdout, script_name)
            sys.exit()
        elif opt in ('-c', '--compress'):
            compress = True
        elif opt in ('-b', '--bytes'):
            try:
                if arg[-1] in 'kK':
                    file_size = int(arg[:-1]) * 1024
                elif arg[-1] in 'mM':
                    file_size = int(arg[:-1]) * 1024 * 1024
                else:
                    file_size = int(arg)
                if file_size < 200 * 1024: raise ValueError()
            except ValueError:
                show_size_error(script_name, arg)
                sys.exit(2)
        elif opt in ('-o', '--output'):
            if os.path.isdir(arg):
                output_dir = arg
            else:
                show_file_error(script_name, arg)
                sys.exit(3)

    if len(args) > 0:
        show_usage(sys.stderr, script_name)
        show_suggestion(sys.stderr, script_name)
        sys.exit(4)

    wiki_extractor = WikiExtractor()
    output_splitter = OutputSplitter(compress, file_size, output_dir)
    process_data(sys.stdin, wiki_extractor, output_splitter)
    output_splitter.close()

if __name__ == '__main__':
    main()



import os

from config import *

def create_dir(name):
    if not os.path.exists(name):
        os.makedirs(name)
        
#Check if dependencies are available
def check():
    #SPARQL server responds
    try:
        from sparql_access import count_entities_of_type
        count_entities_of_type([u'http://dbpedia.org/ontology/Settlement'])
    except ValueError:
        raise RuntimeError('Cannot access SPARQL endpoint: %s.' % sparql_endpoint)
    #raw articles are available
    create_dir(data_path)
    create_dir(raw_articles_path)
    if not os.listdir(raw_articles_path):
        #Wikipedia data archive is available
        try:
            from bz2 import BZ2File
            create_dir(wikidump_path)
            BZ2File(os.path.join(wikidump_path, articles_url.split('/')[-1]))
        except IOError:
            import urllib2
            print 'Downloading Wikipedia pages-articles archive.'
            try:
                u = urllib2.urlopen(articles_url)
            except urllib2.HTTPError:
                raise RuntimeError('Cannot download file: %s.' % articles_url)
            f = open(os.path.join(wikidump_path, articles_url.split('/')[-1]), 'wb')
            f.write(u.read())
            f.close()
            print 'Finished downloading.'
        import extract_dumps
        print 'Extracting raw articles from the archive.'
        extract_dumps.main()
        print 'Finished extracting.'
  



import cPickle as pickle

class Pickler:
    @staticmethod
    def store(object, filename):
        pickle.dump(object, open(filename, 'wb'), -1)

    @staticmethod
    def load(filename):
        return pickle.load(open(filename, 'rb'))



#encoding: utf-8
import sys
import os
import re
import glob
from subprocess import Popen, PIPE
from os.path import join
from os import chdir, getcwd
from string import digits, punctuation
from locale import atof, setlocale, LC_NUMERIC
import lxml.etree as etree
from collections import namedtuple

from config import lang, numeric_predicates, entities_path, raw_articles_path, verbose, spejd_path, use_parser, parser_type, maltparser_path, use_wordnet
from pickler import Pickler
from polish_stop_words import stop_words
from collect_entities import collect_entities

if use_wordnet:
    from nltk.corpus import wordnet as wn
stop_words = set(stop_words)
Word = namedtuple('Word', ['segment', 'lemma', 'tag', 'parse'])
article_sentence_limit = 10
#set ',' as decimal character

def is_numeric(s):
    if s.lower() in ['infinity', 'nan']:
        return False
    try:
        setlocale(LC_NUMERIC, 'pl_PL.UTF-8')
        atof(s)
        return True
    except ValueError:
        return False

class LanguageToolsFactory:
    @staticmethod
    def get_language_tools():
        if lang == 'pl':
            return PolishTools()
        raise NotImplementedError()
        
class LanguageTools:
    def __init__(self):
        self.entities = collect_entities()
        
    def is_entity(self, segment):
        return segment in self.entities


class PolishTools(LanguageTools):
    def __init__(self):
        LanguageTools.__init__(self)

    def parse_disamb_file(self, f):
        article = []
        sentence = []
        for _, elem in etree.iterparse(f):
            if elem.tag == 'chunk' and elem.attrib['type'] == 's':
                article.append(sentence)
                sentence = []
            elif elem.tag == 'tok':
                segment = elem.getchildren()[0].text.encode('utf-8')
                #default lemma and tag
                lemma = segment
                tag = 'ign'
                for interp in elem.iterchildren():
                    if interp.tag == 'lex' and 'disamb' in interp.attrib:
                        lemma = interp.getchildren()[0].text.encode('utf-8')
                        tag = interp.getchildren()[1].text.encode('utf-8')
                        break
                sentence.append(Word(segment, lemma, tag, ''))
        return article
        
    def parse_spejd_file(self, f):
        article = []
        sentence = []
        group_type_stack = []
        for event, elem in etree.iterparse(f, events=('start', 'end')):
            if elem.tag == 'group':
                if event == 'start':
                    group_type_stack.append(elem.attrib['type'])
                else:
                    group_type_stack.pop()
            if event == 'start':
                continue
            if elem.tag == 'chunk' and elem.attrib['type'] == 's':
                article.append(sentence)
                sentence = []
            elif elem.tag == 'tok':
                segment = elem.getchildren()[0].text.encode('utf-8')
                #default lemma and tag
                lemma = segment
                tag = 'ign'
                for interp in elem.iterchildren():
                    if interp.tag == 'lex' and 'disamb' in interp.attrib:
                        lemma = interp.getchildren()[0].text.encode('utf-8')
                        tag = interp.getchildren()[1].text.encode('utf-8')
                        break
                sentence.append(Word(segment, lemma, tag, group_type_stack[-1] if group_type_stack else ''))
        return article
        
    def join_numerals(self, article):
        for j, sentence in enumerate(article):
            #join numerals ('3' ',' '5' becomes '3,5') and '100' '000' becomes '100000'
            i = 0
            while i < len(sentence) - 2:
                if is_numeric(sentence[i].lemma) and sentence[i+1].lemma in '.,' and is_numeric(sentence[i+2].lemma):
                    if sentence[i+1].lemma == '.':
                        new_lemma = sentence[i].lemma + sentence[i+2].lemma
                    else:
                        new_lemma = sentence[i].lemma + ',' + sentence[i+2].lemma
                    sentence[i] = Word(new_lemma, new_lemma, sentence[i].tag, '')
                    sentence = sentence[: i+1] + sentence[i+3 :]
                i += 1
            i = 0
            while i < len(sentence) - 1:
                if is_numeric(sentence[i].lemma) and is_numeric(sentence[i+1].lemma):
                    new_lemma = sentence[i].lemma + sentence[i+1].lemma
                    sentence[i] = Word(new_lemma, new_lemma, sentence[i].tag, '')
                    sentence = sentence[: i+1] + sentence[i+2 :]
                else:
                    i += 1
            #replace occurrences of 'tysiąc' and 'milion' with their numerical values
            #e.g. ('3,5' 'tys' becomes '3500')
            thousand = ['tysiąc', 'tys']
            million = ['mln', 'milion']
            i = 0
            while i < len(sentence) - 1:
                if is_numeric(sentence[i].lemma) and sentence[i+1].lemma in thousand + million:
                    product = 10**3 if sentence[i+1].lemma in thousand else 10**6
                    new_lemma = str(int(round(atof(sentence[i].lemma) * product)))
                    sentence[i] = Word(new_lemma, new_lemma, sentence[i].tag, '')
                    if i < len(sentence) - 2 and sentence[i+1].lemma == 'tys' and sentence[i+2].lemma == '.':
                        sentence = sentence[: i+1] + sentence[i+3 :]
                    else:
                        sentence = sentence[: i+1] + sentence[i+2 :]
                i += 1
            article[j] = sentence
        return article
        
    def correct_lemmas(self, article, link_dictionary):
        '''Corrects lemmatization by using Wikipedia anchor links and some predefined rules.'''
        for i, sentence in enumerate(article):
            for j, word in enumerate(sentence):
                if word.segment in link_dictionary:
                    lemma_suggested_by_link = link_dictionary[word.segment]
                    if word.lemma != lemma_suggested_by_link:
                        article[i][j] = Word(word.segment, lemma_suggested_by_link, word.tag, word.parse)
                if article[i][j].lemma == 'Województwo':
                     article[i][j] = Word(word.segment, 'województwo', word.tag, word.parse)
        for i, sentence in enumerate(article):
            for j, word in enumerate(sentence):
                #if word is in capital letters not at the beginning of a sentence, lemma should also be in capital letters
                if j > 0 and word.segment.decode('utf-8')[0].isupper() and word.lemma.decode('utf-8')[0].islower():
                    article[i][j] = Word(word.segment, word.segment, word.tag, word.parse)
        return article
        
    def prepare_article(self, article, link_dictionary):
        link_dictionary = {
            k.encode('utf-8'): v.encode('utf-8')
            for k, v in link_dictionary.iteritems()
        }
        return self.join_numerals(
            self.correct_lemmas(article, link_dictionary)
        )
        
    def prepare_value(self, value, predicate):
        if predicate in numeric_predicates:
            product = 1000 if 'tys' in value else (10**6 if 'mln' in value else 1)
            #Population value may look '99 2001 Census'. We would like to strip the information about census.
            value = re.sub('\d{4} Census', '', value)
            #Sometimes additional information is provided - it should be stripped.
            v = []
            for c in value:
                if c not in '., ' and not c.isdigit():
                    if v:                
                        break
                else:
                    v.append(c)
            value = ''.join(v)
            try:
                f = atof(filter(lambda c: c.isdigit() or c == ',', value))
                return [str(int(product * f))]
            except ValueError:
                pass
            value = value.split(' ')
            for v in value:
                try:
                    f = atof(v)
                    return [str(int(f))]
                except ValueError:
                    pass
            return []
        else:
            #in Polish DBPedia a picture is often a part of a value 
            #and it is saved as e.g. "20px Neapol" where "Neapol" is the right value
            value = re.sub('\dpx', '', value)
            try:
                value = value.decode('utf-8')
            except UnicodeEncodeError:
                pass
            values = ''.join((c if c.isalpha() else ' ') for c in value).split()
            if len(values) > 1:
                values.append(value)
            values = map(lambda v: v.encode('utf-8'), values)
            values = filter(lambda v: len(v) > 1 and v not in stop_words, values)
            values = filter(lambda v: v not in ['rzeka', 'potok', 'miasto', 'wieś'], values)
            return values
        
    def run_nlptools(self, link_dictionaries):
        if use_parser:
            if parser_type == 'shallow':
                self.run_tagger()
                return self.run_spejd(link_dictionaries)
            elif parser_type == 'dependency':
                return self.run_dependency_parser(self.run_tagger(link_dictionaries))
            raise RuntimeError('Parser type unknown.')
        return self.run_tagger(link_dictionaries)

    def run_tagger(self, link_dictionaries=None):
        '''runs pantera-tagger on all .txt files in raw_articles_path directory and then parses the results'''
        command = 'pantera --skip-done --tagset nkjp -o xces-disamb %s/*.txt' % raw_articles_path
        #Pantera tagger sometimes segmfaults in seemingly random places.
        #Being unable to find the cause, I simply call it again, until it finishes without errors.
        while True:
            p = Popen(command, stdout=PIPE, stdin=PIPE, stderr=PIPE, shell=True)
            _, err = p.communicate()
            if 'All done' in err:
                break
        #if link dictionaries were not provided, don't save the articles yet, only after running the parser
        if link_dictionaries is None:
            return
        articles = {}
        for f in glob.glob('%s/*.disamb' % raw_articles_path):
            try:
                i = int(f[len(raw_articles_path)+1 : -len('.txt.disamg')])
                articles[i] = self.prepare_article(self.parse_disamb_file(f), link_dictionaries[i])
            except ValueError:
                continue
        return articles
        
    def run_spejd(self, link_dictionaries):
        command = 'spejd %s' % raw_articles_path
        old_path = getcwd()
        chdir(spejd_path)
        p = Popen(command, stdout=PIPE, stdin=PIPE, stderr=PIPE, shell=True)
        p.communicate()
        chdir(old_path)
        articles = {}
        for f in glob.glob('%s/*.spejd' % raw_articles_path):
            try:
                i = int(f[len(raw_articles_path)+1 : -len('.txt.spejd')])
                articles[i] = self.prepare_article(self.parse_spejd_file(f), link_dictionaries[i])
            except ValueError:
                continue
        return articles
        
    def run_dependency_parser(self, articles):
        model_filename = 'skladnica_liblinear_stackeager_final.mco'
        input_filename = 'i.conll'
        output_filename = 'o.conll'
        command = 'java -jar maltparser-1.7.1.jar -c %s -i %s -o %s -m parse' % (model_filename, input_filename, output_filename)
        old_path = getcwd()
        chdir(maltparser_path)
        self.save_conll_format(articles, input_filename)
        p = Popen(command, stdout=PIPE, stdin=PIPE, stderr=PIPE, shell=True)
        p.communicate()
        self.read_conll_format(articles, output_filename)
        os.remove(input_filename)
        os.remove(output_filename)
        chdir(old_path)
        return articles
        
    def save_conll_format(self, articles, filename):
        with open(filename, 'w') as f:
            for article in articles.itervalues():
                for sentence in article:
                    for i, word in enumerate(sentence):
                        print >>f, '\t'.join([
                            str(i+1),
                            word.segment,
                            word.lemma,
                            self.get_tag(word.tag),
                            self.get_tag(word.tag),
                            self.get_morph_features(word.tag)
                        ])
                    print >>f
                
    def read_conll_format(self, articles, filename):
        i, j, k = 0, 0, 0
        values = list(articles.keys())
        with open(filename) as f:
            for line in f:
                if line == '\n':
                    j += 1
                    k = 0
                    continue
                line = line.split('\t')
                index = values[i]
                while j == len(articles[index]):
                    i += 1
                    j = 0
                    index = values[i]
                word = articles[index][j][k]
                articles[index][j][k] = Word(word.segment, word.lemma, word.tag, line[7])
                k += 1
        
    def get_simple_tag(self, tag):
        if tag.startswith('subst:'):
            return 'n'
        elif tag.startswith('adj:'):
            return 'a'
        elif tag.startswith('adv:'):
            return 'r'
        elif tag.startswith('fin:') or tag.startswith('praet:'):
            return 'v'
            
    def get_morph_features(self, tag):
        features = tag.split(':')[1:]
        return '|'.join(features) if features else '_'
        
    def get_tag(self, tag):
        return tag.split(':')[0]
        
    def get_gender(self, tag):
        return tag.split(':')[3] if self.get_simple_tag(tag) in ['a', 'n'] else ''
        
    def get_case(self, tag):
        return tag.split(':')[2] if self.get_simple_tag(tag) in ['a', 'n'] else ''
        
    def get_person(self, tag):
        return tag.split(':')[2] if self.get_simple_tag(tag) == 'v' else ''
        
    def get_number(self, tag):
        return tag.split(':')[1] if self.get_simple_tag(tag) in ['a', 'n', 'v'] else ''
        
    def get_aspect(self, tag):
        return tag.split(':')[3] if self.get_simple_tag(tag) == 'v' else ''
        
    def get_hypernym(self, word, level=1):
        def hypernyms(synsets, level):
            if level == 0:
                return []
            ret = []
            for synset in synsets:
                hyper = synset.hypernyms()
                ret += hyper + hypernyms(hyper, level - 1)
            return ret
    
        if not use_wordnet:
            return []
        simple_tag = self.get_simple_tag(word.tag)

        if simple_tag:
            synsets = wn.synsets(word.lemma, simple_tag)
        else:
            synsets = wn.synsets(word.lemma)
        return map(lambda s: s.name, synsets)[0]

#assert PolishTools().prepare_value('ok. 34,5 km', 'd%C5%82ugo%C5%9B%C4%87')[0] == '34'
#assert PolishTools().prepare_value('12,1 tys.', 'populacja')[0] == '12100'
#assert PolishTools().prepare_value('0,9 mln.', 'populacja')[0] == '900000'
#assert PolishTools().prepare_value('warmińsko-mazurskie', 'stolica') == ['warmińsko', 'mazurskie', 'warmińsko-mazurskie']
#assert is_numeric('50,3')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import os
from urllib2 import unquote
from bz2 import BZ2File
import lxml.etree as etree
from pickler import Pickler
from codecs import open as copen
from os.path import join

from config import entities_path, raw_articles_path, wikidump_path, articles_url
from collect_entities import collect_entities

def main():
    entities = collect_entities()
    parse(join(wikidump_path, articles_url.split('/')[-1]), entities, raw_articles_path)
            
def parse(filename, entities, dirname):
    if not os.path.exists(dirname):
        os.makedirs(dirname)
    f = BZ2File(filename)
    for _, elem in etree.iterparse(f, events=('end',)):
        if elem.tag[-5:] == 'title':
            name = elem.text.encode('utf-8').replace(' ', '_')
        elif elem.tag[-4:] == 'text':
            if name in entities:
                #some names have slashes in them
                name = name.replace('/', '_').replace(' ', '_')
                out = copen(join(dirname, name), 'w', 'utf-8')
                print >>out, elem.text
                out.close()
        elem.clear()
    f.close()
    



#encoding: utf-8

from urllib import quote_plus
import os

verbose = 1
save_to_cache = 1
#evaluation mode: 0 -> no evaluation, 1 -> performs evaluation using data from the tests/ directory
evaluation_mode = 1
#limits number of candidates for learning
candidates_limit = 100000000
#limits number of triples used in training
training_limit = 20000
#use Wordnet synsets as features
use_wordnet = False
#use a parser (this increases running time)
use_parser = False
parser_type = 'dependency' #'shallow'

#predicates to learn
predicates = [
    'populacja',
    'stolica',
    'uchodziDo',
    'gmina',
    'powiat',
    'region',
    'prowincja',
    'długość',
    'hrabstwo',
    'powDorzecza',
    'średniPrzepływ',
    'gęstość',
    'powierzchnia',
    'stan',
][2:3]
predicates = map(quote_plus, predicates)
numeric_predicates = set(map(quote_plus, ['populacja', 'długość', 'powDorzecza', 'średniPrzepływ', 'gęstość', 'powierzchnia', 'gęstośćŚrednia', 'szerokość']))

#Some predicates are too broad and it's necessary to specify we are only interested in entities of specific type.
type_restrictions = {
    quote_plus('długość'): 'Stream',
    quote_plus('gęstość'): 'PopulatedPlace',
    quote_plus('szerokość'): 'Stream',
    'powierzchnia': 'PopulatedPlace',
    'stan': 'Settlement',
}

sparql_endpoint = 'http://localhost:8890/sparql/'
articles_url = 'http://dumps.wikimedia.org/plwiki/20110802/plwiki-20110802-pages-articles.xml.bz2'
lang = 'pl'
data_source = "http://dbpedia.org" if lang == 'en' else 'http://%s.dbpedia.org' % lang
main_path = os.path.abspath(os.path.join(os.path.dirname( __file__ ), '..')) + '/'

#Relative paths
data_path = main_path + 'data/'
ext_path = main_path + 'ext/'
raw_articles_path = data_path + '%s/articles' % lang
wikidump_path = data_path + '%s/wiki' % lang
cache_path = main_path + 'cache/'
entities_path = cache_path + '%s/entities.pkl' % lang
synonyms_path = cache_path + '%s/synonyms.pkl' % lang
articles_cache_path = \
    cache_path + '%s/articles_spejd/%%s' % lang if use_parser and parser_type == 'spejd' \
    else cache_path + '%s/dep_articles/%%s' % lang if use_parser and parser_type == 'dependency' \
    else cache_path + '%s/articles/%%s' % lang
candidates_cache_path = cache_path + '%s/candidates/%%s' % lang
models_cache_path = cache_path + '%s/models/%%s' % lang
results_path = main_path + 'results/%s/' % lang
tests_path = main_path + 'tests/%s/' % lang
spejd_path = ext_path + 'spejd-1.3.6'
maltparser_path = ext_path + 'maltparser-1.7.1/'

if __name__ == '__main__':
    import check
    check.check()



#!/usr/bin/env python
import urllib
import json
import sys
from urllib2 import unquote
from collections import defaultdict

from config import data_source, sparql_endpoint

def full_predicate_name(name):
    return '%s/property/%s' % (data_source, name.decode('utf-8'))

def full_resource_name(name):
    return '%s/resource/%s' % (data_source, name.decode('utf-8'))
    
def full_type_name(name):
    return 'http://dbpedia.org/ontology/%s' % name

def strip_url_prefix(s):
    return s[len(data_source) + len('/resource/') : ]

def get_data(query):
    params = {
    	"query": query,
    	"format": "application/json"
    }
    request = urllib.urlencode(params)
    response = urllib.urlopen(sparql_endpoint, request).read()
    return json.loads(response)
    
def get_results(query):
    data = get_data(query)['results']['bindings']
    return [
        unquote(strip_url_prefix(line['s']['value']).encode('utf-8'))
        for line in data
    ]     
    
def get_pairs(query):
    data = get_data(query)['results']['bindings']
    return [
        (unquote(strip_url_prefix(line['s']['value']).encode('utf-8')), line['o']['value'])
        for line in data
    ]     
	
def select_all(d):
    dd = {}
    for c in ['s', 'p', 'o']:
        if c not in d:
            dd[c] = '?%c' % c
        else:
            dd[c] = '<' + d[c] + '>' if c != 'p' else '<' + full_predicate_name(d[c]) + '>'
    query = 'SELECT * FROM <%s> WHERE {%s %s %s} ORDER BY ?s' % (data_source, dd['s'], dd['p'], dd['o'])
    data = get_data(query)['results']['bindings']
    ret = []
    for line in data:
        t = []
        for c in ['s', 'p', 'o']:
            if c in line:
                value = line[c]['value']
                if value.startswith('%s/resource/' % data_source):
                    value = strip_url_prefix(value)
                value = unquote(value.encode('utf-8'))
                t.append(value)
        ret.append(tuple(t))
    return ret
    
def select_types(predicate, subject=True):
    whose_type = '?s' if subject else '?o'
    query = '''SELECT ?s, ?type FROM <%s> WHERE {
          ?s <%s> ?o.
          %s rdf:type ?type.
    }''' % (data_source, full_predicate_name(predicate), whose_type)
    data = get_data(query)['results']['bindings']
    types_dict = defaultdict(list)
    for line in data:
        types_dict[line['s']['value']].append(line['type']['value'])
    return [types for entity, types in types_dict.iteritems()]
    
def count_entities_of_type(type):
    query = '''SELECT count(*) FROM <%s> WHERE {
        ?s a <%s>.
    }''' % (data_source, type)
    return int(get_data(query)['results']['bindings'][0]['callret-0']['value'])
    
def select_entities_of_type(type):
    query = '''SELECT * FROM <%s> WHERE {
        ?s a <%s>.
    }''' % (data_source, type)
    return get_results(query)
    
def select_entities_of_type_not_in_relation(type, predicate):
    #Queries like the one below don't work on Virtuoso version 6.1 (on 6.4 they do).
    #Therefore I use two queries and join their results manually.
    '''SELECT * WHERE {
        {SELECT ?s WHERE { 
            ?s <http://pl.dbpedia.org/property/populacja> ?o. 
        }}
        MINUS
        {{SELECT ?s WHERE { 
            ?s <http://pl.dbpedia.org/property/stolica> ?o. 
        }}}
    }'''
    entities_of_type = select_entities_of_type(type)
    entities_in_relation = set([s for s, o in select_all({'p': predicate})])
    return filter(lambda e: e not in entities_in_relation, entities_of_type)
    
def select_entities_of_type_in_relation(type, predicate):
    query = '''SELECT ?s, ?o FROM <%s> WHERE {
        ?s a <%s>.
        ?s <%s> ?o.
    }''' % (data_source, full_type_name(type), full_predicate_name(predicate))
    return get_pairs(query)
    
def select_all_entities():
    query = '''SELECT DISTINCT ?s FROM <%s> WHERE {
      ?s ?p ?o.
    }''' % data_source
    return get_results(query)
    
if __name__ == '__main__':
    pass

